# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::MachineLearningServices::Mgmt::V2018_03_01_preview
  #
  # These APIs allow end users to operate on Azure Machine Learning Workspace
  # resources.
  #
  class MachineLearningCompute
    include MsRestAzure

    #
    # Creates and initializes a new instance of the MachineLearningCompute class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [MachineLearningServicesClient] reference to the MachineLearningServicesClient
    attr_reader :client

    #
    # Gets computes in specified workspace.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param skiptoken [String] Continuation token for pagination.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<ComputeResource>] operation results.
    #
    def list_by_workspace(resource_group_name, workspace_name, skiptoken:nil, custom_headers:nil)
      first_page = list_by_workspace_as_lazy(resource_group_name, workspace_name, skiptoken:skiptoken, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Gets computes in specified workspace.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param skiptoken [String] Continuation token for pagination.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_workspace_with_http_info(resource_group_name, workspace_name, skiptoken:nil, custom_headers:nil)
      list_by_workspace_async(resource_group_name, workspace_name, skiptoken:skiptoken, custom_headers:custom_headers).value!
    end

    #
    # Gets computes in specified workspace.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param skiptoken [String] Continuation token for pagination.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_workspace_async(resource_group_name, workspace_name, skiptoken:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/computes'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name},
          query_params: {'api-version' => @client.api_version,'$skiptoken' => skiptoken},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::PaginatedComputeResourcesList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets compute definition by its name. Any secrets (storage keys, service
    # credentials, etc) are not returned - use 'keys' nested resource to get them.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ComputeResource] operation results.
    #
    def get(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      response = get_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets compute definition by its name. Any secrets (storage keys, service
    # credentials, etc) are not returned - use 'keys' nested resource to get them.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      get_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
    end

    #
    # Gets compute definition by its name. Any secrets (storage keys, service
    # credentials, etc) are not returned - use 'keys' nested resource to get them.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, 'compute_name is nil' if compute_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/computes/{computeName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'computeName' => compute_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::ComputeResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates or updates compute. This call will overwrite a compute if it exists.
    # This is a nonrecoverable operation. If your intent is to create a new
    # compute, do a GET first to verify that it does not exist yet.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param parameters [ComputeResource] Payload with Machine Learning compute
    # definition.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ComputeResource] operation results.
    #
    def create_or_update(resource_group_name, workspace_name, compute_name, parameters, custom_headers:nil)
      response = create_or_update_async(resource_group_name, workspace_name, compute_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param parameters [ComputeResource] Payload with Machine Learning compute
    # definition.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def create_or_update_async(resource_group_name, workspace_name, compute_name, parameters, custom_headers:nil)
      # Send request
      promise = begin_create_or_update_async(resource_group_name, workspace_name, compute_name, parameters, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::ComputeResource.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Deletes specified Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def delete(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      response = delete_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_async(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      # Send request
      promise = begin_delete_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # System Update On Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def system_update(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      response = system_update_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def system_update_async(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      # Send request
      promise = begin_system_update_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Gets secrets related to Machine Learning compute (storage keys, service
    # credentials, etc).
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ComputeSecrets] operation results.
    #
    def list_keys(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      response = list_keys_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets secrets related to Machine Learning compute (storage keys, service
    # credentials, etc).
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_keys_with_http_info(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      list_keys_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
    end

    #
    # Gets secrets related to Machine Learning compute (storage keys, service
    # credentials, etc).
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_keys_async(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, 'compute_name is nil' if compute_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/computes/{computeName}/listKeys'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'computeName' => compute_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::ComputeSecrets.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates or updates compute. This call will overwrite a compute if it exists.
    # This is a nonrecoverable operation. If your intent is to create a new
    # compute, do a GET first to verify that it does not exist yet.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param parameters [ComputeResource] Payload with Machine Learning compute
    # definition.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ComputeResource] operation results.
    #
    def begin_create_or_update(resource_group_name, workspace_name, compute_name, parameters, custom_headers:nil)
      response = begin_create_or_update_async(resource_group_name, workspace_name, compute_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates or updates compute. This call will overwrite a compute if it exists.
    # This is a nonrecoverable operation. If your intent is to create a new
    # compute, do a GET first to verify that it does not exist yet.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param parameters [ComputeResource] Payload with Machine Learning compute
    # definition.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_create_or_update_with_http_info(resource_group_name, workspace_name, compute_name, parameters, custom_headers:nil)
      begin_create_or_update_async(resource_group_name, workspace_name, compute_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates or updates compute. This call will overwrite a compute if it exists.
    # This is a nonrecoverable operation. If your intent is to create a new
    # compute, do a GET first to verify that it does not exist yet.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param parameters [ComputeResource] Payload with Machine Learning compute
    # definition.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_create_or_update_async(resource_group_name, workspace_name, compute_name, parameters, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, 'compute_name is nil' if compute_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::ComputeResource.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/computes/{computeName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'computeName' => compute_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::ComputeResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::ComputeResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Deletes specified Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_delete(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      response = begin_delete_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Deletes specified Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_with_http_info(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      begin_delete_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
    end

    #
    # Deletes specified Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_async(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, 'compute_name is nil' if compute_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/computes/{computeName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'computeName' => compute_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # System Update On Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_system_update(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      response = begin_system_update_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # System Update On Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_system_update_with_http_info(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      begin_system_update_async(resource_group_name, workspace_name, compute_name, custom_headers:custom_headers).value!
    end

    #
    # System Update On Machine Learning compute.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param compute_name [String] Name of the Azure Machine Learning compute.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_system_update_async(resource_group_name, workspace_name, compute_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, 'compute_name is nil' if compute_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/computes/{computeName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'computeName' => compute_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Gets computes in specified workspace.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PaginatedComputeResourcesList] operation results.
    #
    def list_by_workspace_next(next_page_link, custom_headers:nil)
      response = list_by_workspace_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets computes in specified workspace.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_workspace_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_workspace_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Gets computes in specified workspace.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_workspace_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MachineLearningServices::Mgmt::V2018_03_01_preview::Models::PaginatedComputeResourcesList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets computes in specified workspace.
    #
    # @param resource_group_name [String] Name of the resource group in which
    # workspace is located.
    # @param workspace_name [String] Name of Azure Machine Learning workspace.
    # @param skiptoken [String] Continuation token for pagination.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PaginatedComputeResourcesList] which provide lazy access to pages of
    # the response.
    #
    def list_by_workspace_as_lazy(resource_group_name, workspace_name, skiptoken:nil, custom_headers:nil)
      response = list_by_workspace_async(resource_group_name, workspace_name, skiptoken:skiptoken, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_workspace_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
